/*
 * XdccDownloader - CommandReader.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.ui.stdin;

import static com.google.common.collect.FluentIterable.from;
import static java.util.Arrays.asList;
import static net.pterodactylus.xdcc.ui.stdin.Ansi.bold;
import static net.pterodactylus.xdcc.ui.stdin.Ansi.green;
import static net.pterodactylus.xdcc.ui.stdin.Ansi.red;
import static net.pterodactylus.xdcc.ui.stdin.Command.TO_NAME;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.pterodactylus.irc.util.MessageCleaner;
import net.pterodactylus.xdcc.core.Core;
import net.pterodactylus.xdcc.core.event.DownloadFailed;
import net.pterodactylus.xdcc.core.event.DownloadFinished;
import net.pterodactylus.xdcc.core.event.DownloadStarted;
import net.pterodactylus.xdcc.core.event.GenericMessage;
import net.pterodactylus.xdcc.core.event.MessageReceived;
import net.pterodactylus.xdcc.data.Download;
import net.pterodactylus.xdcc.util.io.DuplicateLineSuppressingWriter;

import com.google.common.base.Joiner;
import com.google.common.base.Optional;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableSet;
import com.google.common.eventbus.Subscribe;
import com.google.common.util.concurrent.AbstractExecutionThreadService;

/**
 * Command interface for arbitrary {@link Reader}s.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class CommandReader extends AbstractExecutionThreadService {

	/** The commands to process. */
	private final Collection<Command> commands;

	/** The reader to read commands from. */
	private final BufferedReader reader;

	/** The writer to write the results to. */
	private final DuplicateLineSuppressingWriter writer;
	private final Collection<Download> failedDownloads;

	/**
	 * Creates a new command reader.
	 *
	 * @param core
	 * 		The core being controlled
	 * @param reader
	 * 		The reader to read commands from
	 * @param writer
	 * 		The write to write results to
	 */
	public CommandReader(Core core, Reader reader, Writer writer, Collection<Download> failedDownloads) {
		this.reader = new BufferedReader(reader);
		this.writer = new DuplicateLineSuppressingWriter(writer);
		this.failedDownloads = failedDownloads;

		/* initialize commands. */
		ImmutableList.Builder<Command> commandBuilder = ImmutableList.builder();
		commandBuilder.add(new ListDownloadsCommand(core));
		commandBuilder.add(new SearchCommand(core));
		commandBuilder.add(new DownloadCommand(core));
		commandBuilder.add(new StatsCommand(core));
		commandBuilder.add(new ListConnectionsCommand(core));
		commandBuilder.add(new AbortDownloadCommand(core));
		commandBuilder.add(new DisconnectCommand(core));
		commandBuilder.add(new FailedDownloadsCommand(failedDownloads));
		commandBuilder.add(new RestartCommand(core, failedDownloads));
		commandBuilder.add(new ResearchCommand(core));
		commands = commandBuilder.build();
	}

	//
	// ABSTRACTEXECUTIONTHREADSERVICE METHODS
	//

	@Override
	protected void run() throws Exception {
		String lastLine = "";
		String line;
		net.pterodactylus.xdcc.ui.stdin.State state = new net.pterodactylus.xdcc.ui.stdin.State();
		while ((line = reader.readLine()) != null) {
			line = line.trim();
			if (line.equals("")) {
				line = lastLine;
			}
			writer.reset();
			String[] words = line.split(" +");
			String commandName = words[0];
			Collection<Command> eligibleCommands = findEligibleCommands(commandName);
			if (eligibleCommands.isEmpty()) {
				writeLine(String.format("Invalid command: %s, valid: %s.", commandName, Joiner.on(' ').join(from(commands).transform(TO_NAME))));
			} else if (eligibleCommands.size() > 1) {
				writeLine(String.format("Commands: %s.", Joiner.on(' ').join(from(eligibleCommands).transform(TO_NAME))));
			} else {
				Command command = eligibleCommands.iterator().next();
				List<String> parameters = from(asList(words)).skip(1).toList();
				state = command.execute(state, parameters, writer);
			}

			lastLine = line;
		}
	}

	//
	// EVENT HANDLERS
	//

	/**
	 * Called when a download was started.
	 *
	 * @param downloadStarted
	 * 		The download started event
	 */
	@Subscribe
	public void downloadStarted(DownloadStarted downloadStarted) {
		Download download = downloadStarted.download();
		try {
			writeLine(String.format("Download of %s (from %s, %s) has started.", bold(download.pack().name()), download.bot().name(), download.bot().network().name()));
		} catch (IOException ioe1) {
			/* ignore. */
		}
	}

	/**
	 * Called when a download is finished.
	 *
	 * @param downloadFinished
	 * 		The download finished event
	 */
	@Subscribe
	public void downloadFinished(DownloadFinished downloadFinished) {
		Download download = downloadFinished.download();
		removeFailedDownloads(download.pack().name());
		try {
			writeLine(green(String.format("Download of %s (from %s, %s) has finished, at %s/s.", download.pack().name(), download.bot().name(), download.bot().network().name(), f(download.dccReceiver().overallRate()))));
		} catch (IOException ioe1) {
			/* ignore. */
		}
	}

	private void removeFailedDownloads(String name) {
		List<Download> failedDownloadsToRemove = new ArrayList<>();
		for (Download failedDownload : failedDownloads) {
			if (failedDownload.pack().name().equals(name)) {
				failedDownloadsToRemove.add(failedDownload);
			}
		}
		failedDownloads.removeAll(failedDownloadsToRemove);
	}

	/**
	 * Called when a download fails.
	 *
	 * @param downloadFailed
	 * 		The download failed event
	 */
	@Subscribe
	public void downloadFailed(DownloadFailed downloadFailed) {
		Download download = downloadFailed.download();
		failedDownloads.add(download);
		try {
			writeLine(red(String.format("Download of %s (from %s, %s) has failed at %.1f%% and %s/s.", download.filename(), download.bot().name(), download.bot().network().name(), download.dccReceiver().progress() * 100.0 / download.dccReceiver().size(), f(download.dccReceiver().overallRate()))));
		} catch (IOException ioe1) {
			/* ignore. */
		}
	}

	/**
	 * Displays the received message on the console.
	 *
	 * @param messageReceived
	 * 		The message received event
	 */
	@Subscribe
	public void messageReceived(MessageReceived messageReceived) {
		try {
			writeLine(String.format("Message from %s: %s", messageReceived.source(), MessageCleaner.getDefaultInstance().clean(messageReceived.message())));
		} catch (IOException e) {
			/* ignore. */
		}
	}

	/**
	 * Writes a generic message to the console.
	 *
	 * @param genericMessage
	 * 		The generic message event
	 */
	@Subscribe
	public void genericMessage(GenericMessage genericMessage) {
		try {
			writeLine(genericMessage.message());
		} catch (IOException ioe1) {
			/* ignore. */
		}
	}

	//
	// PRIVATE METHODS
	//

	private Collection<Command> findEligibleCommands(String name) {
		ImmutableSet.Builder<Command> eligibleCommands = ImmutableSet.builder();
		for (Command command : commands) {
			if (command.getName().toLowerCase().startsWith(name.toLowerCase())) {
				eligibleCommands.add(command);
			}
			for (String alias : command.getAliases()) {
				if (alias.toLowerCase().startsWith(name.toLowerCase())) {
					eligibleCommands.add(command);
				}
			}
		}
		return eligibleCommands.build();
	}

	/**
	 * Writes the given line followed by an LF to the {@link #writer}.
	 *
	 * @param line
	 * 		The line to write
	 * @throws IOException
	 * 		if an I/O error occurs
	 */
	private void writeLine(String line) throws IOException {
		writer.write(line + "\n");
		writer.flush();
	}

	/**
	 * Converts large numbers into a human-friendly format, by showing SI prefixes
	 * for ×1024 (K), ×1048576 (M), and ×1073741824 (G).
	 *
	 * @param number
	 * 		The number to convert
	 * @return The converted number
	 */
	static String f(long number) {
		if (number >= (1 << 30)) {
			return String.format("%.1fG", number / (double) (1 << 30));
		}
		if (number >= (1 << 20)) {
			return String.format("%.1fM", number / (double) (1 << 20));
		}
		if (number >= (1 << 10)) {
			return String.format("%.1fK", number / (double) (1 << 10));
		}
		return String.format("%dB", number);
	}

	/**
	 * Formats the given number of seconds into a more easily readable string.
	 *
	 * @param seconds
	 * 		The number of seconds
	 * @return The formatted time, or “unknown” if the time is unknown
	 */
	static String t(Optional<Long> seconds) {
		if (!seconds.isPresent()) {
			return "unknown";
		}
		if (seconds.get() > 3600) {
			return String.format("%02d:%02d:%02d", seconds.get() / 3600, (seconds.get() / 60) % 60, seconds.get() % 60);
		}
		return String.format("%02d:%02d", (seconds.get() / 60) % 60, seconds.get() % 60);
	}

}
