/*
 * XdccDownloader - ListDownloadsCommand.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.ui.stdin;

import static com.google.common.collect.FluentIterable.from;
import static com.google.common.collect.Lists.newArrayList;
import static java.util.Arrays.asList;
import static net.pterodactylus.xdcc.data.Download.BY_NAME;
import static net.pterodactylus.xdcc.data.Download.BY_RUNNING;
import static net.pterodactylus.xdcc.data.Download.SECONDS_LEFT;
import static net.pterodactylus.xdcc.ui.stdin.CommandReader.f;
import static net.pterodactylus.xdcc.ui.stdin.CommandReader.t;

import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.List;

import net.pterodactylus.irc.DccReceiver;
import net.pterodactylus.xdcc.core.Core;
import net.pterodactylus.xdcc.data.Download;

import com.google.common.collect.FluentIterable;
import com.google.common.collect.Ordering;

/**
 * Command that will list all current downloads.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class ListDownloadsCommand implements Command {

	private static final int PROGRESS_BAR_WIDTH = 10;

	/** The core to operate on. */
	private final Core core;

	/**
	 * Creates a new list downloads command.
	 *
	 * @param core
	 * 		The core to operate on
	 */
	public ListDownloadsCommand(Core core) {
		this.core = core;
	}

	//
	// COMMAND METHODS
	//

	@Override
	public String getName() {
		return "list";
	}

	@Override
	public Collection<String> getAliases() {
		return asList("dcc");
	}

	@Override
	public State execute(State state, List<String> parameters, Writer outputWriter) throws IOException {
		int counter = 0;
		List<Download> downloads = newArrayList(from(core.downloads()).toSortedList(Ordering.from(BY_RUNNING).compound(BY_NAME)));
		for (Download download : downloads) {
			DccReceiver dccReceiver = download.dccReceiver();
			if (dccReceiver == null) {
						/* download has not even started. */
				outputWriter.write(String.format("[%d] %s requested from %s (not started yet)\n", counter++, download.pack().name(), download.bot().name()));
				continue;
			}
			outputWriter.write(String.format("[%d] %s %s from %s (%s, ", counter++, getProgressBar(dccReceiver, PROGRESS_BAR_WIDTH), dccReceiver.filename(), download.bot().name(), f(dccReceiver.size())));
			if (dccReceiver.isRunning()) {
				outputWriter.write(String.format("%.1f%%, %s/s, %s", dccReceiver.progress() * 100.0 / dccReceiver.size(), f(dccReceiver.currentRate()), t(SECONDS_LEFT.apply(download))));
			} else {
				if (dccReceiver.progress() >= dccReceiver.size()) {
					outputWriter.write(String.format("complete, %s/s", f(dccReceiver.overallRate())));
				} else {
					outputWriter.write(String.format("aborted at %.1f%%, %s/s", dccReceiver.progress() * 100.0 / dccReceiver.size(), f(dccReceiver.currentRate())));
				}
			}
			outputWriter.write(")\n");
		}
		outputWriter.write("End of DCCs.\n");
		outputWriter.flush();
		return state.setLastDownloads(downloads);
	}

	/**
	 * Creates a progress bar for the given DCC receiver.
	 *
	 * @param dccReceiver
	 * 		The DCC receiver to create the progress bar for
	 * @param progressBarWidth
	 * 		The width of the progress bar (in characters)
	 * @return The progress bar for the given DCC receiver
	 */
	private static String getProgressBar(DccReceiver dccReceiver, int progressBarWidth) {
		FluentIterable<Character> partialProgressCharacters = from(asList(' ', '\u258f', '\u258e', '\u258d', '\u258c', '\u258b', '\u258a', '\u2589', '\u2588'));
		double progress = dccReceiver.progress() * 100.0 / dccReceiver.size();
		double singleBlockWidth = 100.0 / progressBarWidth;
		int fullProgressBlocks = (int) (progress / singleBlockWidth);
		double lastBlockProgress = (progress - fullProgressBlocks * singleBlockWidth) / singleBlockWidth;
		StringBuilder progressBar = new StringBuilder(progressBarWidth);
		for (int i = 0; i < progressBarWidth; ++i) {
			if (i < fullProgressBlocks) {
				progressBar.append(partialProgressCharacters.last().get());
			} else if (i > fullProgressBlocks) {
				progressBar.append(partialProgressCharacters.first().get());
			} else {
				progressBar.append(partialProgressCharacters.get((int) (lastBlockProgress * (partialProgressCharacters.size() - 1))));
			}
		}
		return progressBar.toString();
	}

}
